/*=====================================================================

  File:      spHtmGeoRemove.sql for Spatial Sample
  Summary:   Implements the Transact-SQL interface to the Spatial library
			 Removes geographical lat/lon functions ONLY. Assumes, that
		     spHtmCsharp.sql has already been run.
  Date:	     August 19, 2005

---------------------------------------------------------------------
  This spatial search library was developed by Alexander Szalay, Robert Brunner, 
  Peter Kunszt, and Gyorgy Fekete at the Department of Physics and Astronomy, 
  The Johns Hopkins University, in collaboration with Jim Gray, Microsoft Research.
  Details of the algorithms can be found at http://www.sdss.jhu.edu/htm/.
 ======================================================= */

--=================================================================================
-- spHtmGeoRemove.sql 
--   Gyorgy Fekete, Aug 2006
-----------------------------------------------------------------------------------
-- Removes Geographical (lat/lon based) functions.
--  
-- Removes: 
--		fHtmLatLon
--		fHtmXyzToLatLon
--		fHtmLatLonToXyz
--      fDistanceLatLon
--		fHtmCoverCircleLatLon
--      fHtmNearbyLatLon		-- nearby objects
--      fHtmNearestLatLon		-- nearest object
----------------------------------------------------------------------------------
-- History: (see spHtmCsharp.sql)

----------------------------------------------------------------------------------
--* todo:   
----------------------------------------------------------------------------------

SET NOCOUNT ON
go
exec sp_configure 'clr enabled', 1
reconfigure with override
go
if exists (select * from sys.objects where [name] = N'fHtmLatLon') 
			drop function  fHtmLatLon
if exists (select * from sys.objects where [name] = N'fHtmXyzToLatLon') 
			drop function fHtmXyzToLatLon
if exists (select * from sys.objects where [name] = N'fHtmLatLonToXyz')
			drop function fHtmLatLonToXyz
if exists (select * from sys.objects where [name] = N'fDistanceLatLon') 
			drop function fDistanceLatLon
if exists (select * from sys.objects where [name] = N'fDistanceXyz') 
			drop function fDistanceXyz
if exists (select * from sys.objects where [name] = N'fHtmCoverCircleLatLon') 
			drop function  fHtmCoverCircleLatLon
if exists (select * from sys.objects where [name] = N'fHtmNearbyLatLon') 
			drop function fHtmNearbyLatLon
if exists (select * from sys.objects where [name] = N'fHtmNearestLatLon') 
			drop function fHtmNearestLatLon
--


declare @version varchar(max)
select @version = dbo.fHtmVersion()
PRINT '[spHtmCsharp.sql]: Version: ' + @version + ' of the HTM library successfully altered: Geo (lat/lon) functions removed'
GO
